'use strict';

var gulp = require('gulp-help')(require('gulp'));
var runSequence = require('run-sequence');
var _ = require('lodash');

var finalTasks = {
    bundle: []
};

function run(settings) {

    _.forEach(settings, function(options, configTaskName) {
        loadTask(configTaskName, settings);
    });

    return createTasks(settings);
}

function loadTask(configTaskName, settings) {
    var enableInBundle = _enableInBundle(configTaskName);
    var taskName = _parseTaskName(configTaskName);
    var taskOptions = settings[configTaskName];
    var task =  require('./tasks/' + taskName)(taskOptions);

    finalTasks[taskName] = {
        desc: task.desc,
        dependencies: task.dependencies || false,
        subtasks: []
    };

    if (enableInBundle) {
        finalTasks.bundle.push(taskName);
    }

    task.subtasks.forEach(function(subtask){
        var enableSubtask = _enbleSubtask(subtask, taskOptions);

        if (enableSubtask) {
            finalTasks[taskName].subtasks.push(subtask);
        }
    });
}

function createTasks(settings) {

    var bundle = finalTasks.bundle;
    delete finalTasks.bundle;

    _.forEach(finalTasks, function(task, taskName) {
        if (task.subtasks.length) {
            var selectedSubtasks = [];

            task.subtasks.forEach(function(subtask) {
                var gulpTaskName = [taskName, subtask.name].join(':');
                var enableInTaskBundle = _enableInTaskBundle(subtask, settings[taskName] || settings['_'+taskName]);

                subtask = _parseSubtaskOptions(subtask, enableInTaskBundle);

                gulp.task(gulpTaskName, subtask.desc, subtask.callback);

                if ( enableInTaskBundle ) {
                    selectedSubtasks.push(gulpTaskName);
                }
            });

            if (task.dependencies) {
                selectedSubtasks = (task.dependencies.exec === 'after')
                    ? selectedSubtasks.concat(task.dependencies.tasks)
                    : task.dependencies.tasks.concat(selectedSubtasks);
            }

            gulp.task(taskName, task.desc, function(done) {
                selectedSubtasks.push(done);
                runSequence.apply({}, selectedSubtasks);
            });
        }
    });

    if (bundle.length) {
        gulp.task('build', 'Run tasks: ' + bundle.join(', '), bundle);
    }

}

function _enableInBundle(name) {
    return name.indexOf('_');
}

function _parseTaskName(name) {
    if (!name.indexOf('_')) {
        name = name.substr(1);
    }

    return name;
}

function _enbleSubtask(subtask, options) {

    if (subtask.forceRun) {
        return true;
    }

    if (!options._tasks) {
        return true;
    } else {
        var exists = (options._tasks[options._tasks.type].indexOf(subtask.name) > -1);
        return (options._tasks.type === 'enable') ? exists : !exists;
    }
}

function _parseSubtaskOptions(task, inBundle) {
    task.desc = (inBundle) ? false : task.desc || "";
    return task;
}

function _enableInTaskBundle(subtask, options) {
    if (subtask.forceBundle !== undefined) {
        return subtask.forceBundle;
    }

    if (options._tasks === undefined || options._tasks.noBundle === undefined) {
        return true;
    } else if ( options._tasks.noBundle.indexOf(subtask.name) > -1 ) {
        return false;
    } else {
        return true;
    }
}

/////////////////////////////

module.exports = run;
